@@ -4,7 +4,7 @@
 /*                                                                         */
 /*    FreeType font driver interface (specification).                      */
 /*                                                                         */
-/*  Copyright 1996-2001, 2002 by                                           */
+/*  Copyright 1996-2001, 2002, 2003 by                                     */
 /*  David Turner, Robert Wilhelm, and Werner Lemberg.                      */
 /*                                                                         */
 /*  This file is part of the FreeType project, and may only be used,       */
@@ -169,9 +169,9 @@ FT_BEGIN_HEADER
   {
     FT_Module_Class           root;
 
-    FT_Int                    face_object_size;
-    FT_Int                    size_object_size;
-    FT_Int                    slot_object_size;
+    FT_Long                   face_object_size;
+    FT_Long                   size_object_size;
+    FT_Long                   slot_object_size;
 
     FT_Face_InitFunc          init_face;
     FT_Face_DoneFunc          done_face;
