@@ -4,7 +4,7 @@
 /*                                                                         */
 /*    FreeType PFR object methods (body).                                  */
 /*                                                                         */
-/*  Copyright 2002 by                                                      */
+/*  Copyright 2002, 2003 by                                                */
 /*  David Turner, Robert Wilhelm, and Werner Lemberg.                      */
 /*                                                                         */
 /*  This file is part of the FreeType project, and may only be used,       */
@@ -20,6 +20,7 @@
 #include "pfrload.h"
 #include "pfrgload.h"
 #include "pfrcmap.h"
+#include "pfrsbit.h"
 #include FT_OUTLINE_H
 #include FT_INTERNAL_DEBUG_H
 
@@ -40,20 +41,33 @@
   FT_LOCAL_DEF( void )
   pfr_face_done( PFR_Face  face )
   {
+    FT_Memory  memory = face->root.driver->root.memory;
+
+
+    /* we don't want dangling pointers */
+    face->root.family_name = NULL;
+    face->root.style_name  = NULL;
+    
     /* finalize the physical font record */
     pfr_phy_font_done( &face->phy_font, FT_FACE_MEMORY( face ) );
 
     /* no need to finalize the logical font or the header */
+    FT_FREE( face->root.available_sizes );
   }
 
 
   FT_LOCAL_DEF( FT_Error )
-  pfr_face_init( FT_Stream  stream,
-                 PFR_Face   face,
-                 FT_Int     face_index )
+  pfr_face_init( FT_Stream      stream,
+                 PFR_Face       face,
+                 FT_Int         face_index,
+                 FT_Int         num_params,
+                 FT_Parameter*  params )
   {
     FT_Error  error;
 
+    FT_UNUSED( num_params );
+    FT_UNUSED( params );
+
 
     /* load the header and check it */
     error = pfr_header_load( &face->header, stream );
@@ -100,93 +114,127 @@
       goto Exit;
 
     /* now load the physical font descriptor */
-     error = pfr_phy_font_load( &face->phy_font, stream,
-                                 face->log_font.phys_offset,
-                                 face->log_font.phys_size );
-     if ( error )
-       goto Exit;
+    error = pfr_phy_font_load( &face->phy_font, stream,
+                               face->log_font.phys_offset,
+                               face->log_font.phys_size );
+    if ( error )
+      goto Exit;
 
-     /* now, set-up all root face fields */
-     {
-       FT_Face      root     = FT_FACE( face );
-       PFR_PhyFont  phy_font = &face->phy_font;
+    /* now, set-up all root face fields */
+    {
+      FT_Face      root     = FT_FACE( face );
+      PFR_PhyFont  phy_font = &face->phy_font;
 
 
-       root->face_index = face_index;
-       root->num_glyphs = phy_font->num_chars;
-       root->face_flags = FT_FACE_FLAG_SCALABLE;
+      root->face_index = face_index;
+      root->num_glyphs = phy_font->num_chars;
+      root->face_flags = FT_FACE_FLAG_SCALABLE;
 
-       if ( (phy_font->flags & PFR_PHY_PROPORTIONAL) == 0 )
-         root->face_flags |= FT_FACE_FLAG_FIXED_WIDTH;
+      if ( (phy_font->flags & PFR_PHY_PROPORTIONAL) == 0 )
+        root->face_flags |= FT_FACE_FLAG_FIXED_WIDTH;
 
-       if ( phy_font->flags & PFR_PHY_VERTICAL )
-         root->face_flags |= FT_FACE_FLAG_VERTICAL;
-       else
-         root->face_flags |= FT_FACE_FLAG_HORIZONTAL;
+      if ( phy_font->flags & PFR_PHY_VERTICAL )
+        root->face_flags |= FT_FACE_FLAG_VERTICAL;
+      else
+        root->face_flags |= FT_FACE_FLAG_HORIZONTAL;
 
-       /* XXX: kerning and embedded bitmap support isn't there yet */
+      if ( phy_font->num_strikes > 0 )
+        root->face_flags |= FT_FACE_FLAG_FIXED_SIZES;
 
-       root->family_name = phy_font->font_id;
-       root->style_name  = NULL;  /* no style name in font file */
+      if ( phy_font->num_kern_pairs > 0 )
+        root->face_flags |= FT_FACE_FLAG_KERNING;
 
-       root->num_fixed_sizes = 0;
-       root->available_sizes = 0;
+      /* If no family name was found in the "undocumented" auxiliary
+       * data, use the font ID instead.  This sucks but is better than
+       * nothing.
+       */
+      root->family_name = phy_font->family_name;
+      if ( root->family_name == NULL )
+        root->family_name = phy_font->font_id;
 
-       root->bbox         = phy_font->bbox;
-       root->units_per_EM = (FT_UShort)phy_font->outline_resolution;
-       root->ascender     = (FT_Short) phy_font->bbox.yMax;
-       root->descender    = (FT_Short) phy_font->bbox.yMin;
-       root->height       = (FT_Short)
-                              ( ( ( root->ascender - root->descender ) * 12 )
-                                / 10 );
+      /* note that the style name can be NULL in certain PFR fonts,
+       * probably meaning "Regular"
+       */
+      root->style_name = phy_font->style_name;
 
-       /* now compute maximum advance width */
-       if ( ( phy_font->flags & PFR_PHY_PROPORTIONAL ) == 0 )
-         root->max_advance_width = (FT_Short)phy_font->standard_advance;
-       else
-       {
-         FT_Int    max = 0;
-         FT_UInt   count = phy_font->num_chars;
-         PFR_Char  gchar = phy_font->chars;
+      root->num_fixed_sizes = 0;
+      root->available_sizes = 0;
+
+      root->bbox         = phy_font->bbox;
+      root->units_per_EM = (FT_UShort)phy_font->outline_resolution;
+      root->ascender     = (FT_Short) phy_font->bbox.yMax;
+      root->descender    = (FT_Short) phy_font->bbox.yMin;
+      root->height       = (FT_Short)(
+        ( ( root->ascender - root->descender ) * 12 ) / 10 );
+
+      if ( phy_font->num_strikes > 0 )
+      {
+        FT_UInt          n, count = phy_font->num_strikes;
+        FT_Bitmap_Size*  size;
+        PFR_Strike       strike;
+        FT_Memory        memory = root->stream->memory;
+         
+         
+        if ( FT_NEW_ARRAY( root->available_sizes, count ) )
+          goto Exit;
+         
+        size   = root->available_sizes;
+        strike = phy_font->strikes;
+        for ( n = 0; n < count; n++, size++, strike++ )
+        {
+          size->height = (FT_UShort)strike->y_ppm;
+          size->width  = (FT_UShort)strike->x_ppm;
+        }
+        root->num_fixed_sizes = count;
+      }
+
+      /* now compute maximum advance width */
+      if ( ( phy_font->flags & PFR_PHY_PROPORTIONAL ) == 0 )
+        root->max_advance_width = (FT_Short)phy_font->standard_advance;
+      else
+      {
+        FT_Int    max = 0;
+        FT_UInt   count = phy_font->num_chars;
+        PFR_Char  gchar = phy_font->chars;
 
 
-         for ( ; count > 0; count--, gchar++ )
-         {
-           if ( max < gchar->advance )
-             max = gchar->advance;
-         }
+        for ( ; count > 0; count--, gchar++ )
+        {
+          if ( max < gchar->advance )
+            max = gchar->advance;
+        }
 
-         root->max_advance_width = (FT_Short)max;
-       }
+        root->max_advance_width = (FT_Short)max;
+      }
 
-       root->max_advance_height = root->height;
+      root->max_advance_height = root->height;
 
-       root->underline_position  = (FT_Short)( - root->units_per_EM / 10 );
-       root->underline_thickness = (FT_Short)(   root->units_per_EM / 30 );
+      root->underline_position  = (FT_Short)( - root->units_per_EM / 10 );
+      root->underline_thickness = (FT_Short)(   root->units_per_EM / 30 );
 
-       /* create charmap */
-       {
-         FT_CharMapRec  charmap;
+      /* create charmap */
+      {
+        FT_CharMapRec  charmap;
 
 
-         charmap.face        = root;
-         charmap.platform_id = 3;
-         charmap.encoding_id = 1;
-         charmap.encoding    = FT_ENCODING_UNICODE;
+        charmap.face        = root;
+        charmap.platform_id = 3;
+        charmap.encoding_id = 1;
+        charmap.encoding    = FT_ENCODING_UNICODE;
 
-         FT_CMap_New( &pfr_cmap_class_rec, NULL, &charmap, NULL );
+        FT_CMap_New( &pfr_cmap_class_rec, NULL, &charmap, NULL );
 
 #if 0
-         /* Select default charmap */
-         if (root->num_charmaps)
-           root->charmap = root->charmaps[0];
+        /* Select default charmap */
+        if (root->num_charmaps)
+          root->charmap = root->charmaps[0];
 #endif
-       }
+      }
 
-       /* check whether we've loaded any kerning pairs */
-       if ( phy_font->num_kern_pairs )
-         root->face_flags |= FT_FACE_FLAG_KERNING;
-     }
+      /* check whether we've loaded any kerning pairs */
+      if ( phy_font->num_kern_pairs )
+        root->face_flags |= FT_FACE_FLAG_KERNING;
+    }
 
   Exit:
     return error;
@@ -223,7 +271,7 @@
   pfr_slot_load( PFR_Slot  slot,
                  PFR_Size  size,
                  FT_UInt   gindex,
-                 FT_Int    load_flags )
+                 FT_Int32  load_flags )
   {
     FT_Error     error;
     PFR_Face     face    = (PFR_Face)slot->root.face;
@@ -231,12 +279,27 @@
     FT_Outline*  outline = &slot->root.outline;
     FT_ULong     gps_offset;
 
-    if (gindex > 0)
+
+    if ( gindex > 0 )
       gindex--;
 
     /* check that the glyph index is correct */
     FT_ASSERT( gindex < face->phy_font.num_chars );
 
+    /* try to load an embedded bitmap */
+    if ( ( load_flags & ( FT_LOAD_NO_SCALE | FT_LOAD_NO_BITMAP ) ) == 0 )
+    {
+      error = pfr_slot_load_bitmap( slot, size, gindex );
+      if ( error == 0 )
+        goto Exit;
+    }
+
+    if ( load_flags & FT_LOAD_SBITS_ONLY )
+    {
+      error = PFR_Err_Invalid_Argument;
+      goto Exit;
+    }
+
     gchar               = face->phy_font.chars + gindex;
     slot->root.format   = FT_GLYPH_FORMAT_OUTLINE;
     outline->n_points   = 0;
@@ -290,6 +353,21 @@
       metrics->vertBearingX = 0;
       metrics->vertBearingY = 0;
 
+      /* Apply the font matrix, if any.                 */
+      /* TODO: Test existing fonts with unusual matrix  */
+      /* whether we have to adjust Units per EM.        */
+      {
+        FT_Matrix font_matrix;
+
+
+        font_matrix.xx = face->log_font.matrix[0] << 8;
+        font_matrix.yx = face->log_font.matrix[1] << 8;
+        font_matrix.xy = face->log_font.matrix[2] << 8;
+        font_matrix.yy = face->log_font.matrix[3] << 8;
+
+        FT_Outline_Transform( outline, &font_matrix );
+      }
+
       /* scale when needed */
       if ( scaling )
       {
@@ -320,6 +398,7 @@
       metrics->horiBearingY = cbox.yMax - metrics->height;
     }
 
+  Exit:
     return error;
   }
 
@@ -332,54 +411,87 @@
   /*************************************************************************/
   /*************************************************************************/
 
-  /* XXX: This relies on the font being spec-conformant, i.e., that the
-          kerning pairs are sorted.  We might want to sort it just to make
-          sure */
-
-#undef  PFR_KERN_INDEX
-#define PFR_KERN_INDEX( g1, g2 )  ( ( (FT_ULong)g1 << 16 ) | g2 )
-
-  /* find the kerning for a given glyph pair */
-  FT_LOCAL_DEF( FT_Error )
+  FT_LOCAL_DEF( void )
   pfr_face_get_kerning( PFR_Face    face,
                         FT_UInt     glyph1,
                         FT_UInt     glyph2,
                         FT_Vector*  kerning )
   {
+    FT_Error      error    = PFR_Err_Ok;
     PFR_PhyFont   phy_font = &face->phy_font;
-    PFR_KernPair  min, mid, max;
-    FT_ULong      idx = PFR_KERN_INDEX( glyph1, glyph2 );
+    PFR_KernItem  item     = phy_font->kern_items;
+    FT_UInt32     idx      = PFR_KERN_INDEX( glyph1, glyph2 );
 
+    FT_UNUSED( error ); /* just needed as syntactical sugar */
 
-    /* simple binary search */
-    min = phy_font->kern_pairs;
-    max = min + phy_font->num_kern_pairs;
-
-    while ( min < max )
-    {
-      FT_ULong  midi;
-
-
-      mid  = min + ( max - min ) / 2;
-      midi = PFR_KERN_INDEX( mid->glyph1, mid->glyph2 );
-
-      if ( midi == idx )
-      {
-        *kerning = mid->kerning;
-        goto Exit;
-      }
-
-      if ( midi < idx )
-        min = mid + 1;
-      else
-        max = mid;
-    }
 
     kerning->x = 0;
     kerning->y = 0;
 
-  Exit:
-    return 0;
+    /* find the kerning item containing our pair */
+    while ( item )
+    {
+      if ( item->pair1 <= idx && idx <= item->pair2 )
+        goto Found_Item;
+
+      item = item->next;
+    }
+
+    /* not found */
+    return;
+
+  Found_Item:
+    {
+      /* perform simply binary search within the item */
+      FT_UInt    min, mid, max;
+      FT_Stream  stream = face->root.stream;
+      FT_Byte*   p;
+
+
+      if ( FT_STREAM_SEEK( item->offset )                       ||
+           FT_FRAME_ENTER( item->pair_count * item->pair_size ) )
+        return;
+
+      min = 0;
+      max = item->pair_count;
+      while ( min < max )
+      {
+        FT_UInt  char1, char2, charcode;
+
+
+        mid = ( min + max ) >> 1;
+        p   = stream->cursor + mid*item->pair_size;
+
+        if ( item->flags & PFR_KERN_2BYTE_CHAR )
+        {
+          char1 = FT_NEXT_USHORT( p );
+          char2 = FT_NEXT_USHORT( p );
+        }
+        else
+        {
+          char1 = FT_NEXT_USHORT( p );
+          char2 = FT_NEXT_USHORT( p );
+        }
+        charcode = PFR_KERN_INDEX( char1, char2 );
+
+        if ( idx == charcode )
+        {
+          if ( item->flags & PFR_KERN_2BYTE_ADJ )
+            kerning->x = item->base_adj + FT_NEXT_SHORT( p );
+          else
+            kerning->x = item->base_adj + FT_NEXT_CHAR( p );
+
+          break;
+        }
+        if ( idx > charcode )
+          min = mid + 1;
+        else
+          max = mid;
+      }
+
+      FT_FRAME_EXIT();
+    }
   }
 
+
 /* END */
